unit DtCaDb97;

(******************************************************************************
TDbCalendar97

Derived from
    tCalendar97

Properties
    DataSource, DataField : Database informations
    ReadOnly : Can modify field

Author name=BOURMAD Mehdi
Author E-mail=bourmad@multimania.com
Author URL=www.multimania.com/bourmad
******************************************************************************)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Dialogs, Forms, StdCtrls, Buttons, Db, DbCtrls, DbTables,
  DtCal97, DtMisc97;

type
  TDbCalendar97 = class(TCalendar97)
  private
    FDataLink: TFieldDataLink;
    FCanvas: TControlCanvas;
    FAlignment: TAlignment;
    FFocused: Boolean;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    function GetDate: TDateTime;
    procedure SetDate(dtArg: TDateTime);
    procedure DateChange(Sender: TObject);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
    property Date: TDateTime read GetDate write SetDate;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
  end;

implementation

{-------------------------------------------------------------}
{-------------------- TDbCalendar97 --------------------------}
{-------------------------------------------------------------}
constructor TDbCalendar97.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  OnDateChange := DateChange;
end;

destructor TDbCalendar97.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TDbCalendar97.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
     (AComponent = DataSource) then DataSource := nil;
end;

function TDbCalendar97.GetDate: TDateTime;
begin
  GetDate := inherited Date;
end;

procedure TDbCalendar97.SetDate(dtArg: TDateTime);
begin
  inherited SetDate(dtArg);
  if FDataLink.DataSet.CanModify
  and (FDataLink.Field.AsDateTime <> dtArg)
  then begin
    if dtArg<>0
      then FDataLink.Field.AsDateTime := dtArg
      else FDataLink.Field.Clear;
    FDataLink.Modified;
  end;
end;

procedure TDbCalendar97.DateChange(Sender: TObject);
var
  aDate : tDateTime;
begin
  if FDataLink.Field <> nil then
  begin
    aDate := GetDate;
    FDataLink.Edit;
    SetDate (aDate);
  end;
end;

procedure TDbCalendar97.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
  begin
    inherited Date := FDataLink.Field.AsDateTime;
  end;
end;

procedure TDbCalendar97.CMExit(var Message: TCMExit);
begin
  FDataLink.UpdateRecord;
  inherited;
end;

function TDbCalendar97.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TDbCalendar97.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

function TDbCalendar97.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TDbCalendar97.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TDbCalendar97.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TDbCalendar97.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TDbCalendar97.GetField: TField;
begin
  Result := FDataLink.Field;
end;

end.

