unit DtEdFT97;

(******************************************************************************
TDateFromTo97 

Derived from
    tCustomPanel

Properties
    TabOrder, BevelInner, BevelOuter,
    BorderStyle, BorderWidth : Derived from tCustomPanel
    DateFrom, DateTo : Dates selected
    PanelOrientation : select type of panel (horizontal or vertical)
    TextFrom : Caption 'From'
    TextTo : Caption 'To'
    and all properties from DateEdit97

procedures
    ClearDate: Clear dates from DateFrom and DateTo

Event
    OnChangeFrom : when DateFrom is changed
    OnChangeTo   : when DateTo is changed

Author name=BOURMAD Mehdi
Author E-mail=bourmad@multimania.com
Author URL=www.multimania.com/bourmad
******************************************************************************)

interface

uses
  WinTypes, WinProcs, Buttons, Classes, Controls, ExtCtrls, Forms, Graphics,
  Messages, StdCtrls, SysUtils, DtEdit97, DtMisc97;


type
  TOrientation = (oHorizontal, oVertical);

  TDateFromTo97 = class(TCustomPanel)
  private
    { Private declarations }
    FAbout : TAboutMeProperty;
    FDateFrom,
    FDateTo   : TDateEdit97;
    FDayName : TDayName;
    FLabelFrom,
    FLabelTo   : TLabel;
    FMonthName : TMonthName;
    FOrientation : tOrientation;
    procedure PlaceComponents;
    function GetOnChangeFrom : tNotifyEvent;
    procedure SetOnChangeFrom (value : tNotifyEvent);
    function GetOnChangeTo : tNotifyEvent;
    procedure SetOnChangeTo (value : tNotifyEvent);
    procedure SetOrientation (value : tOrientation);
    function GetLabelFrom : string;
    function GetLabelTo : string;
    procedure SetLabelFrom (value : string);
    procedure SetLabelTo (value : string);
    function GetDateFrom : tDateTime;
    function GetDateTo : tDateTime;
    procedure SetDateFrom (value : tDateTime);
    procedure SetDateTo (value : tDateTime);
    function GetColorCalendar : tCalendarColors;
    function GetColorError : tColor;
    procedure SetColorCalendar (value : tCalendarColors);
    procedure SetColorError (value : tColor);
    function GetBtnDateStyle : tDateBtnStyle;
    procedure SetBtnDateStyle (value : tDateBtnStyle);
    function GetBtnBorderStyle : tBorderStyle;
    procedure SetBtnBorderStyle (value : tBorderStyle);
    function GetBtnStyle : tButtonStyle;
    procedure SetBtnStyle (value : tButtonstyle);
    function GetOptions : TDateOptions;
    procedure SetOptions (value : tDateOptions);
    procedure DayNameChange(ADayName : TObject);
    procedure MonthNameChange(AMonthName : TObject);
    function GetShortCuts : tButtonsShortCut;
    procedure SetShortCuts (value : tButtonsShortCut);
    function GetStartOfWeek : tDayOfWeek;
    procedure SetStartOfWeek (value : tDayOfWeek);
    function GetStartOfWeekEnd : tDayOfWeek;
    procedure SetStartOfWeekEnd (value : tDayOfWeek);
    procedure ValidateDateFromTo(Sender: TObject);
  protected
    { Protected declarations }
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  public
    { Public declarations }
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
    Procedure Loaded; override;
    procedure ClearDate (Sender: TObject);
    property DTEdFrom: TDateEdit97 read FDateFrom write FDateFrom;
    property DTEdTo: TDateEdit97 read FDateTo write FDateTo;
  published
    { Published declarations }
    Property BevelInner;
    Property BevelOuter;
    Property BorderStyle;
    Property BorderWidth;
    Property Font;
    Property TabOrder;
    Property TabStop;
    property About: TAboutMeProperty read FAbout write FAbout;
    property ButtonBorderStyle: tBorderStyle read GetBtnBorderStyle write SetBtnBorderStyle default bsSingle;
    property ButtonDateStyle: tDateBtnStyle read GetBtnDateStyle write SetBtnDateStyle default btsCalendar;
    property ButtonStyle : tButtonStyle read GetBtnStyle write SetBtnStyle default bsAutodetect;
    property ColorCalendar : tCalendarColors read GetColorCalendar write SetColorCalendar;
    property ColorError : tColor read GetColorError write SetColorError default clRed;
    property DateFrom: tDateTime read GetDateFrom write SetDateFrom;
    property DateTo: tDateTime read GetDateTo write SetDateTo;
    property DayNames: TDayName read FDayName write FDayName;
    property MonthNames: TMonthName read FMonthName write FMonthName;
    property Options : tDateOptions read GetOptions write SetOptions;
    property ShortCuts: tButtonsShortCut read GetShortCuts write SetShortCuts;
    property StartOfWeek: TDayOfWeek read GetStartOfWeek write SetStartOfWeek default 1;
    property StartOfWeekEnd: TDayOfWeek read GetStartOfWeekEnd write SetStartOfWeekEnd default 6;
    property OnChangeFrom : TNotifyEvent read GetOnChangeFrom write SetOnChangeFrom;
    property OnChangeTo : TNotifyEvent read GetOnChangeTo write SetOnChangeTo;
    property PanelOrientation : tOrientation read FOrientation write SetOrientation default oHorizontal;
    property TextFrom: String read GetLabelFrom write SetLabelFrom;
    property TextTo: String read GetLabelTo write SetLabelTo;
  end;

implementation

Constructor TDateFromTo97.Create(AOwner:TComponent);
Begin
  inherited Create(AOwner);

  ParentFont := true;

  FDayName := TDayName.Create;
  FDayName.OnChange := DayNameChange;
  FMonthName := TMonthName.Create;
  FMonthName.OnChange := MonthNameChange;

  FLabelFrom:=TLabel.Create(Self);
  FLabelFrom.Parent := Self;
  FLabelFrom.Caption:=IdsFrom;

  FDateFrom:=TDateEdit97.Create(Self);
  FDateFrom.Parent := Self;
  FDateFrom.Date := Date;
  FDateFrom.OnValidateClick := ValidateDateFromTo;

  FLabelTo:=TLabel.Create(Self);
  FLabelTo.Parent := Self;
  FLabelTo.Caption:=IdsTo;

  FDateTo:=TDateEdit97.Create(Self);
  FDateTo.Parent := Self;
  FDateTo.Date := Date;
  FDateTo.OnValidateClick := ValidateDateFromTo;

  ControlStyle := ControlStyle - [csSetCaption];

  PanelOrientation := oHorizontal;
  PlaceComponents;
End;

Procedure TDateFromTo97.Loaded;
Begin
  inherited Loaded;
  FDateFrom.Font := Font;
  FDateFrom.Loaded;
  FDateTo.Font := Font;
  FDateTo.Loaded;
  PlaceComponents;
  Caption:='';
  FDateTo.Parent := Self;
end;

destructor TDateFromTo97.Destroy;
begin
  FDayName.Free;
  FMonthName.Free;
  FDateFrom.Free;
  FDateTo.Free;
  FLabelFrom.Free;
  FLabelTo.Free;
  inherited Destroy;
end;

procedure TDateFromTo97.CMEnabledChanged(var Message: TMessage);
begin
  FDateFrom.Enabled := Enabled;
  FDateTo.Enabled := Enabled;
  Invalidate;
end;

procedure TDateFromTo97.ClearDate (Sender: TObject);
begin
  FDateFrom.ClearDate (Sender);
  FDateTo.ClearDate (Sender);
end;

procedure TDateFromTo97.PlaceComponents;
begin
  Case FOrientation of
    oHorizontal : begin
                    FLabelFrom.Top := 8;
                    FLabelFrom.Left := 8;
                    FDateFrom.Top := 8;
                    FDateFrom.Left := FLabelFrom.Left+FLabelFrom.Width+8;
                    FLabelTo.Top := 8;
                    FLabelTo.Left := FDateFrom.Left+FDateFrom.Width+8;
                    FDateTo.Top := 8;
                    FDateTo.Left := FLabelTo.Left+FLabelTo.Width+8;
                  end;
    oVertical   : begin
                    FLabelFrom.Top := 8;
                    FLabelFrom.Left := 8;
                    FDateFrom.Top := FLabelFrom.Top+FLabelFrom.Height+8;
                    FDateFrom.Left := 16;
                    FLabelTo.Top := FDateFrom.Top+FDateFrom.Height+8;
                    FLabelTo.Left := 8;
                    FDateTo.Top := FLabelTo.Top+FLabelTo.Height+8;
                    FDateTo.Left := 16;
                  end;
  end;
  Width := FDateTo.Left+FDateTo.Width+8;
  Height := FDateTo.Top+FDateTo.Height+8;
end;

function TDateFromTo97.GetOnChangeFrom : tNotifyEvent;
begin
  Result := FDateFrom.Onchange;
end;

procedure TDateFromTo97.SetOnChangeFrom (value : tNotifyEvent);
begin
  FDateFrom.OnChange := value;
end;

function TDateFromTo97.GetOnChangeTo : tNotifyEvent;
begin
  Result := FDateTo.Onchange;
end;

procedure TDateFromTo97.SetOnChangeTo (value : tNotifyEvent);
begin
  FDateTo.OnChange := value;
end;

procedure TDateFromTo97.SetOrientation (value : tOrientation);
begin
  if FOrientation <> value
  then begin
    FOrientation := value;
    PlaceComponents;
  end;
end;

function TDateFromTo97.GetLabelFrom : string;
begin
  Result := FLabelFrom.Caption;
end;

function TDateFromTo97.GetLabelTo : string;
begin
  Result := FLabelTo.Caption;
end;

procedure TDateFromTo97.SetLabelFrom (value : string);
begin
  FLabelFrom.Caption := value;
  PlaceComponents;
end;

procedure TDateFromTo97.SetLabelTo (value : string);
begin
  FLabelTo.Caption := value;
  PlaceComponents;
end;

function TDateFromTo97.GetDateFrom : tDateTime;
begin
  Result := FDateFrom.Date;
end;

function TDateFromTo97.GetDateTo : tDateTime;
begin
  Result := FDateTo.Date;
end;

procedure TDateFromTo97.SetDateFrom (value : tDateTime);
begin
  FDateFrom.Date := value;
  Update;
end;

procedure TDateFromTo97.SetDateTo (value : tDateTime);
begin
  FDateTo.Date := value;
  Update;
end;

function TDateFromTo97.GetColorCalendar : tCalendarColors;
begin
  Result := FDateFrom.ColorCalendar;
end;

function TDateFromTo97.GetColorError : tColor;
begin
  Result := FDateFrom.ColorError;
end;

procedure TDateFromTo97.SetColorCalendar (value : tCalendarColors);
begin
  FDateFrom.ColorCalendar := value;
  FDateTo.ColorCalendar := value;
  PlaceComponents;
end;

procedure TDateFromTo97.SetColorError (value : tColor);
begin
  FDateFrom.ColorError := value;
  FDateTo.ColorError := value;
  PlaceComponents;
end;

function TDateFromTo97.GetBtnBorderStyle : tBorderStyle;
begin
  Result := FDateFrom.BorderStyle;
end;

procedure TDateFromTo97.SetBtnBorderStyle (value : tBorderStyle);
begin
  FDateFrom.BorderStyle := value;
  FDateTo.BorderStyle := value;
end;

function TDateFromTo97.GetBtnDateStyle : tDateBtnStyle;
begin
  Result := FDateFrom.ButtonDateStyle;
end;

procedure TDateFromTo97.SetBtnDateStyle (value : tDateBtnStyle);
begin
  FDateFrom.ButtonDateStyle := value;
  FDateTo.ButtonDateStyle := value;
end;

function TDateFromTo97.GetBtnStyle : tButtonstyle;
begin
  Result := FDateFrom.ButtonStyle;
end;

procedure TDateFromTo97.SetBtnStyle (value : tButtonstyle);
begin
  FDateFrom.ButtonStyle := value;
  FDateTo.ButtonStyle := value;
end;

function TDateFromTo97.GetOptions : TDateOptions;
begin
  Result := FDateFrom.Options;
end;

procedure TDateFromTo97.SetOptions (value : tDateOptions);
begin
  FDateFrom.Options := value;
  FDateTo.Options := value;
end;

procedure TDateFromTo97.DayNameChange(ADayName : TObject);
begin
  FDayName.Assign(ADayName as TDayName);
  FDateFrom.DayNames.Assign(ADayName as TDayName);
  FDateFrom.DayNames.OnChange (FDayName);
  FDateTo.DayNames.Assign(ADayName as TDayName);
  FDateTo.DayNames.OnChange (FDayName);
end;

procedure TDateFromTo97.MonthNameChange(AMonthName : TObject);
begin
  FMonthName.Assign(AMonthName as TMonthName);
  FDateFrom.MonthNames.Assign(AMonthName as TMonthName);
  FDateFrom.MonthNames.OnChange (FMonthName);
  FDateTo.MonthNames.Assign(AMonthName as TMonthName);
  FDateFrom.MonthNames.OnChange (FMonthName);
end;

function TDateFromTo97.GetShortCuts : tButtonsShortCut;
begin
  Result := FDateFrom.ShortCuts;
end;

procedure TDateFromTo97.SetShortCuts (value : tButtonsShortCut);
begin
  FDateFrom.ShortCuts := value;
  FDateTo.ShortCuts := value;
end;

function TDateFromTo97.GetStartOfWeek : tDayOfWeek;
begin
  Result := FDateFrom.StartOfWeek;
end;

procedure TDateFromTo97.SetStartOfWeek (value : tDayOfWeek);
begin
  FDateFrom.StartOfWeek := value;
  FDateTo.StartOfWeek := value;
end;

function TDateFromTo97.GetStartOfWeekEnd : tDayOfWeek;
begin
  Result := FDateFrom.StartOfWeekEnd;
end;

procedure TDateFromTo97.SetStartOfWeekEnd (value : tDayOfWeek);
begin
  FDateFrom.StartOfWeekEnd := value;
  FDateTo.StartOfWeekEnd := value;
end;

procedure TDateFromTo97.ValidateDateFromTo(Sender: TObject);
var
  ParentComponent: TComponent;
begin
{We must find first the topmost Parent Object (The Form)
 to perform the sendmessage.. and voila..}
{$IFDEF VER80}
{  ParentComponent := TDateEdit97(Sender as TBitBtn).Owner;
  while not (ParentComponent.Owner=nil) do
    ParentComponent := ParentComponent.Owner;
  SendMessage((ParentComponent as TForm).Handle, WM_NEXTDLGCTL, 0, 0);}
{$ELSE}
  if TComponent(Sender).Owner <> nil
  then begin
    ParentComponent := TComponent(Sender).Owner.GetParentComponent;
    repeat
      ParentComponent := ParentComponent.GetParentComponent;
    until not ParentComponent.HasParent;
    SendMessage((ParentComponent as TForm).Handle, WM_NEXTDLGCTL, 0, 0);
  end;
{$ENDIF}
end;

end.

